from rocolib.api.components import Component
from rocolib.api.composables.graph.Face import Face, Rectangle, RegularNGon as Shape
from rocolib.api.composables import GraphComposable as Graph
from rocolib.api.ports.EdgePort import EdgePort
import rocolib.utils.numsym as math

class PaddlewheelR(Component):

    _test_params = {
        'Radius': 60,
        'Spokes': 8,
        'PanelLength': 45,
    }

    def define(self):
        self.addParameter("Radius", 60, paramtype="length")
        self.addParameter("Spokes", 8, paramtype="count")
        self.addParameter("PanelLength", 45, paramtype="length")
        for i in range(12):
            self.addInterface("flap%dR"%i, EdgePort(self, None, "PanelLength"))

    def assemble(self):
        R = self.getParameter("Radius")
        S = self.getParameter("Spokes")
        P = self.getParameter("PanelLength")

        # XXX TODO: add min and max values to Parameter objects
        p = max(P, R/3)
        p = min(p, R-10)
        s = max(S, 4)
        s = min(s, 12)
        r = R-p
        an = 180./s
        cl = 2*r*math.sin(math.deg2rad(an))
        x = math.sin(math.deg2rad(an))*p
        y = x*math.cos(math.deg2rad(an))
        b = x*math.sin(math.deg2rad(an))
        a = y*math.tan(math.deg2rad(90-2*an))

        graph = Graph()
        graph.addFace(Shape("", s, cl), "center")

        for i in range(0, s):
             h = Face("", ((p,0), (cl*math.sin(math.deg2rad(an))+p, cl*math.cos(math.deg2rad(an))), (cl*math.sin(math.deg2rad(an)), cl*math.cos(math.deg2rad(an))), (0,0)))
             graph.attachEdge("center.e%d" % i, h, "e1", "leg%d" % i, angle=0)
             g = Face("", ((0,0), (p, 0), (p-b, y), (a,y)))
             graph.attachEdge("leg%d.e0" % i, g, "e1", "flap%dR" % i, angle=90)
             self.setInterface("flap%dR"%i, EdgePort(self, "flap%dR.e3"%i, "PanelLength"))


        self.composables["graph"] = graph

if __name__ == "__main__":
    h = PaddlewheelR()
    h.test()